@@ -50,6 +50,9 @@ Par2Repairer::Par2Repairer(void)
   outputbuffer = 0;
 
   noiselevel = CommandLine::nlNormal;
+
+  skipdata = false;
+  skipleaway = 0;
 }
 
 Par2Repairer::~Par2Repairer(void)
@@ -83,13 +86,19 @@ Result Par2Repairer::Process(const CommandLine &commandline, bool dorepair)
   // What noiselevel are we using
   noiselevel = commandline.GetNoiseLevel();
 
+  // Should we skip data whilst scanning files
+  skipdata = commandline.GetSkipData();
+
+  // How much leaway should we allow when scanning files
+  skipleaway = commandline.GetSkipLeaway();
+
   // do we want to purge par files on success ?
   bool purgefiles = commandline.GetPurgeFiles();
 
-  // Get filesnames from the command line
-  string par2filename = commandline.GetParFilename();
-  string basepath = commandline.GetBasePath();
-  const list<CommandLine::ExtraFile> &extrafiles = commandline.GetExtraFiles();
+  // Get filenames from the command line
+  std::string par2filename = commandline.GetParFilename();
+  std::string basepath = commandline.GetBasePath();
+  std::list<CommandLine::ExtraFile> extrafiles = commandline.GetExtraFiles();
 
   // Determine the searchpath from the location of the main PAR2 file
   string name;
@@ -140,10 +149,10 @@ Result Par2Repairer::Process(const CommandLine &commandline, bool dorepair)
     cout << endl << "Verifying source files:" << endl << endl;
 
   // Attempt to verify all of the source files
-  if (!VerifySourceFiles(basepath))
+  if (!VerifySourceFiles(basepath, extrafiles))
     return eFileIOError;
 
-  if (completefilecount<mainpacket->RecoverableFileCount())
+  if (completefilecount < mainpacket->RecoverableFileCount())
   {
     if (noiselevel > CommandLine::nlQuiet)
       cout << endl << "Scanning extra files:" << endl << endl;
@@ -164,7 +173,7 @@ Result Par2Repairer::Process(const CommandLine &commandline, bool dorepair)
     return eRepairNotPossible;
 
   // Are any of the files incomplete
-  if (completefilecount<mainpacket->RecoverableFileCount())
+  if (completefilecount < mainpacket->RecoverableFileCount())
   {
     // Do we want to carry out a repair
     if (dorepair)
@@ -177,7 +186,7 @@ Result Par2Repairer::Process(const CommandLine &commandline, bool dorepair)
         return eFileIOError;
 
       // Are we still missing any files
-      if (completefilecount<mainpacket->RecoverableFileCount())
+      if (completefilecount < mainpacket->RecoverableFileCount())
       {
         // Work out which files are being repaired, create them, and allocate
         // target DataBlocks to them, and remember them for later verification.
@@ -1083,7 +1092,7 @@ static bool SortSourceFilesByFileName(Par2RepairerSourceFile *low,
 }
 
 // Attempt to verify all of the source files
-bool Par2Repairer::VerifySourceFiles(string basepath)
+bool Par2Repairer::VerifySourceFiles(const std::string& basepath, std::list<CommandLine::ExtraFile>& extrafiles)
 {
   bool finalresult = true;
 
@@ -1130,14 +1139,27 @@ bool Par2Repairer::VerifySourceFiles(string basepath)
     Par2RepairerSourceFile *sourcefile = *sf;
 
     // What filename does the file use
-    string filename = sourcefile->TargetFileName();
+    const std::string& file = sourcefile->TargetFileName();
+    const std::string& name = DiskFile::SplitRelativeFilename(file, basepath);
+    const std::string& target_pathname = DiskFile::GetCanonicalPathname(file);
+    
+    // if the target file is in the list of extra files, we remove it
+    // from the extra files.
+    list<CommandLine::ExtraFile>::iterator it = extrafiles.begin();
+    for (; it != extrafiles.end(); ++it)
+    {
+      const CommandLine::ExtraFile& e = *it;
+      const std::string& extra_pathname = e.FileName();
+      if (!extra_pathname.compare(target_pathname))
+      {
+        extrafiles.erase(it);
+        break;
+      }
+    }
 
     // Check to see if we have already used this file
-    if (diskFileMap.Find(filename) != 0)
+    if (diskFileMap.Find(file) != 0)
     {
-      string name;
-      DiskFile::SplitRelativeFilename(filename, basepath, name);
-
       // The file has already been used!
       cerr << "Source file " << name << " is a duplicate." << endl;
 
@@ -1148,7 +1170,7 @@ bool Par2Repairer::VerifySourceFiles(string basepath)
       DiskFile *diskfile = new DiskFile;
 
       // Does the target file exist
-      if (diskfile->Open(filename))
+      if (diskfile->Open(file))
       {
         // Yes. Record that fact.
         sourcefile->SetTargetExists(true);
@@ -1173,9 +1195,6 @@ bool Par2Repairer::VerifySourceFiles(string basepath)
 
         if (noiselevel > CommandLine::nlSilent)
         {
-          string name;
-          DiskFile::SplitRelativeFilename(filename, basepath, name);
-
           cout << "Target: \"" << name << "\" - missing." << endl;
         }
       }
@@ -1477,8 +1496,23 @@ bool Par2Repairer::ScanDataFile(DiskFile                *diskfile,    // [in]
 
   u64 progress = 0;
 
-  u32 noduplicatenonext = 0;
-  u32 noduplicatenonextmax = 10;
+  // How far will we scan the file (1 byte at a time)
+  // before skipping ahead looking for the next block
+  u64 scandistance = min(skipleaway<<1, blocksize);
+
+  // Distance to skip forward if we don't find a block
+  u64 scanskip = skipdata ? blocksize - scandistance : 0;
+
+  // Assume with are half way through scanning
+  u64 scanoffset = scandistance >> 1;
+
+  // Total number of bytes that were skipped whilst scanning
+  u64 skippeddata = 0;
+
+  // Offset of last data that was found
+  u64 lastmatchoffset = 0;
+
+  bool progressline = false;
 
   // Whilst we have not reached the end of the file
   while (filechecksummer.Offset() < diskfile->FileSize())
@@ -1491,6 +1525,8 @@ bool Par2Repairer::ScanDataFile(DiskFile                *diskfile,    // [in]
       if (oldfraction != newfraction)
       {
         cout << "Scanning: \"" << shortname << "\": " << newfraction/10 << '.' << newfraction%10 << "%\r" << flush;
+
+	progressline = true;
       }
     }
 
@@ -1504,6 +1540,18 @@ bool Par2Repairer::ScanDataFile(DiskFile                *diskfile,    // [in]
     // Did we find a match
     if (currententry != 0)
     {
+      if (lastmatchoffset < filechecksummer.Offset() && noiselevel > CommandLine::nlNormal)
+      {
+	if (progressline)
+	{
+	  cout << endl;
+	  progressline = false;
+	}
+
+	cout << "No data found between offset " << lastmatchoffset
+	     << " and " << filechecksummer.Offset() << endl;
+      }
+
       // Is this the first match
       if (count == 0)
       {
@@ -1550,6 +1598,12 @@ bool Par2Repairer::ScanDataFile(DiskFile                *diskfile,    // [in]
       // Advance to the next block
       if (!filechecksummer.Jump(currententry->GetDataBlock()->GetLength()))
         return false;
+
+      // If the next match fails, assume we hare half way through scanning for the next block
+      scanoffset = scandistance >> 1;
+
+      // Update offset of last match
+      lastmatchoffset = filechecksummer.Offset();
     }
     else
     {
@@ -1557,7 +1611,7 @@ bool Par2Repairer::ScanDataFile(DiskFile                *diskfile,    // [in]
       matchtype = ePartialMatch;
 
       // Was this a duplicate match
-      if (duplicate)
+      if (duplicate && false) // ignore duplicates
       {
         duplicatecount++;
 
@@ -1573,30 +1627,49 @@ bool Par2Repairer::ScanDataFile(DiskFile                *diskfile,    // [in]
         // What entry do we expect next
         nextentry = 0;
 
-        if (noduplicatenonext < noduplicatenonextmax)
-        {
-          if (!filechecksummer.Step())
-            return false;
+        if (!filechecksummer.Step())
+          return false;
 
-          noduplicatenonext++;
-        }
-        else
-        {
-          if (!filechecksummer.Jump((blocksize - noduplicatenonext)))
-            return false;
+	u64 skipfrom = filechecksummer.Offset();
 
-          noduplicatenonext = 0;
-        }
+	// Have we scanned too far without finding a block?
+	if (scanskip > 0
+	    && ++scanoffset >= scandistance
+	    && skipfrom < diskfile->FileSize())
+	{
+	  // Skip forwards to where we think we might find more data
+	  if (!filechecksummer.Jump(scanskip))
+	    return false;
+
+	  // Update the count of skipped data
+	  skippeddata += filechecksummer.Offset() - skipfrom;
+
+	  // Reset scan offset to 0
+	  scanoffset = 0;
+	}
       }
     }
   }
 
+  if (lastmatchoffset < filechecksummer.Offset() && noiselevel > CommandLine::nlNormal)
+  {
+    if (progressline)
+    {
+      cout << endl;
+      progressline = false;
+    }
+
+    cout << "No data found between offset " << lastmatchoffset
+         << " and " << filechecksummer.Offset() << endl;
+  }
+
   // Get the Full and 16k hash values of the file
   filechecksummer.GetFileHashes(hashfull, hash16k);
 
   if (noiselevel >= CommandLine::nlDebug)
   {
-    cout << "duplicates: " << duplicatecount << endl;
+    if (duplicatecount > 0)
+      cout << "duplicates: " << duplicatecount << endl;
     cout << "matchcount: " << count << endl;
   }
 
@@ -1686,6 +1759,13 @@ bool Par2Repairer::ScanDataFile(DiskFile                *diskfile,    // [in]
                  << endl;
           }
         }
+
+	if (skippeddata > 0)
+        {
+	  cout << skippeddata << " bytes of data were skipped whilst scanning." << endl
+	       << "If there are not enough blocks found to repair: try again "
+	       << "with the -N option." << endl;
+	}
       }
     }
     else
@@ -1749,6 +1829,13 @@ bool Par2Repairer::ScanDataFile(DiskFile                *diskfile,    // [in]
              << "\" - no data found." 
              << endl;
       }
+
+      if (skippeddata > 0)
+      {
+	cout << skippeddata << " bytes of data were skipped whilst scanning." << endl
+	     << "If there are not enough blocks found to repair: try again "
+	     << "with the -N option." << endl;
+      }
     }
   }
 
