@@ -70,7 +70,7 @@ protected:
   bool ComputeWindowTable(void);
 
   // Attempt to verify all of the source files
-  bool VerifySourceFiles(string basepath);
+  bool VerifySourceFiles(const std::string& basepath, std::list<CommandLine::ExtraFile>& extrafiles);
 
   // Scan any extra files specified on the command line
   bool VerifyExtraFiles(const list<CommandLine::ExtraFile> &extrafiles, string basepath);
@@ -130,6 +130,9 @@ protected:
 
   string                    searchpath;              // Where to find files on disk
 
+  bool                      skipdata;                // Should we skip data whilst scanning
+  u64                       skipleaway;              // The leaway +/- we should allow whilst scanning
+
   bool                      firstpacket;             // Whether or not a valid packet has been found.
   MD5Hash                   setid;                   // The SetId extracted from the first packet.
 
